@@ -31,7 +31,6 @@
 #include "include/core/SkImageGenerator.h"
 #include "include/core/SkImageInfo.h"
 #include "include/core/SkM44.h"
-#include "include/core/SkMatrix44.h"
 #include "include/core/SkMaskFilter.h"
 #include "include/core/SkPaint.h"
 #include "include/core/SkPath.h"
@@ -62,6 +61,7 @@
 // effects/
 #include "include/effects/Sk1DPathEffect.h"
 #include "include/effects/Sk2DPathEffect.h"
+#include "include/effects/SkBlenders.h"
 #include "include/effects/SkColorMatrix.h"
 #include "include/effects/SkColorMatrixFilter.h"
 #include "include/effects/SkCornerPathEffect.h"
@@ -88,8 +88,8 @@
 // utils/
 #include "include/utils/SkCamera.h"
 #include "include/utils/SkCustomTypeface.h"
-#include "include/utils/SkInterpolator.h"
 #include "include/utils/SkNullCanvas.h"
+#include "include/utils/SkOrderedFontMgr.h"
 #include "include/utils/SkParsePath.h"
 #include "include/utils/SkShadowUtils.h"
 #include "include/utils/SkTextUtils.h"
@@ -172,6 +172,20 @@ extern "C" void C_SkEncodedOriginToMatrix(SkEncodedOrigin origin, int w, int h,
 
 extern "C" void C_Core_Types(SkGraphics *, SkCoverageMode *, SkColorChannelFlag *) {};
 
+//
+// core/SkBlender.h
+//
+
+extern "C" SkBlender* C_SkBlender_Mode(SkBlendMode mode) {
+    return SkBlender::Mode(mode).release();
+}
+
+extern "C" SkBlender* C_SkBlender_Deserialize(const void* data, size_t length) {
+    return static_cast<SkBlender*>(SkFlattenable::Deserialize(SkFlattenable::kSkBlender_Type, data, length).release());
+}
+
+
+
 //
 // core/SkCubicMap.h
 //
@@ -438,8 +452,18 @@ extern "C" void C_SkPaint_setColorFilter(SkPaint* self, SkColorFilter* colorFilt
     self->setColorFilter(sp(colorFilter));
 }
 
-extern "C" SkBlendMode C_SkPaint_getBlendMode(const SkPaint* self) {
-    return self->getBlendMode();
+extern "C" bool C_SkPaint_asBlendMode(const SkPaint* self, SkBlendMode* mode) {
+    auto r = self->asBlendMode();
+    if (r.has_value()) {
+        *mode = *r;
+        return true;
+    } else {
+        return false;
+    }
+}
+
+extern "C" void C_SkPaint_setBlender(SkPaint* self, SkBlender* blender) {
+    self->setBlender(sp(blender));
 }
 
 extern "C" void C_SkPaint_setPathEffect(SkPaint* self, SkPathEffect* pathEffect) {
@@ -831,6 +855,10 @@ extern "C" bool C_SkM44_equals(const SkM44 *self, const SkM44 *other) {
     return *self == *other;
 }
 
+extern "C" void C_SkM44_RectToRect(const SkRect* src, const SkRect* dst, SkM44* uninitialized) {
+    new(uninitialized) SkM44(SkM44::RectToRect(*src, *dst));
+}
+
 extern "C" void C_SkM44_LookAt(const SkV3* eye, const SkV3* center, const SkV3* up, SkM44* uninitialized) {
     new(uninitialized) SkM44(SkM44::LookAt(*eye, *center, *up));
 }
@@ -847,36 +875,6 @@ extern "C" SkV4 C_SkM44_map(const SkM44* self, float x, float y, float z, float
     return self->map(x, y, z, w);
 }
 
-//
-// SkMatrix44
-//
-
-extern "C" void C_SkMatrix44_ConstructIdentity(SkMatrix44* uninitialized) {
-    new(uninitialized) SkMatrix44(SkMatrix44::kIdentity_Constructor);
-}
-
-extern "C" void C_SkMatrix44_ConstructNaN(SkMatrix44* uninitialized) {
-    new(uninitialized) SkMatrix44(SkMatrix44::kNaN_Constructor);
-}
-
-// SkMatrix44_Equals is not generated by bindgen.
-extern "C" bool C_SkMatrix44_Equals(const SkMatrix44* self, const SkMatrix44* rhs) {
-    return *self == *rhs;
-}
-
-// SkMatrix44_SkMatrix conversion.
-extern "C" void C_SkMatrix44_SkMatrix(const SkMatrix44* self, SkMatrix* m) {
-    *m = SkMatrix(*self);
-}
-
-extern "C" void C_SkMatrix44_Mul(const SkMatrix44* self, const SkMatrix44* rhs, SkMatrix44* result) {
-    *result = *self * *rhs;
-}
-
-extern "C" void C_SkMatrix44_MulV4(const SkMatrix44* self, const SkVector4* rhs, SkVector4* result) {
-    *result = *self * *rhs;
-}
-
 //
 // core/SkMatrix.h
 //
@@ -1283,6 +1281,10 @@ extern "C" void C_SkFont_ConstructFromTypefaceWithSizeScaleAndSkew(SkFont* unini
     new(uninitialized) SkFont(sp(typeface), size, scaleX, skewX);
 }
 
+extern "C" void C_SkFont_destruct(SkFont* self) {
+    self->~SkFont();
+}
+
 extern "C" bool C_SkFont_Equals(const SkFont* self, const SkFont* other) {
     return *self == *other;
 }
@@ -1307,8 +1309,16 @@ extern "C" void C_SkFont_setTypeface(SkFont* self, SkTypeface* tf) {
     self->setTypeface(sp(tf));
 }
 
-extern "C" void C_SkFont_destruct(SkFont* self) {
-    self->~SkFont();
+extern "C" void C_SkFont_getIntercepts(
+    const SkFont* self, 
+    const SkGlyphID glyphs[], 
+    int count, 
+    const SkPoint pos[], 
+    SkScalar top, SkScalar bottom, 
+    const SkPaint* paint, 
+    VecSink<SkScalar>* vs) {
+    auto r = self->getIntercepts(glyphs, count, pos, top, bottom, paint);
+    vs->set(r);
 }
 
 //
@@ -1524,9 +1534,7 @@ extern "C" SkColorFilter* C_SkColorFilter_makeComposed(const SkColorFilter* self
 }
 
 extern "C" SkColorFilter* C_SkColorFilter_Deserialize(const void* data, size_t length) {
-    // TODO: there is no "official" Deserialize wrapper in SkColorFilter, so we
-    //       are not sure if deserialization is supported at all.
-    return static_cast<SkColorFilter*>(SkFlattenable::Deserialize(SkFlattenable::kSkColorFilter_Type, data, length).release());
+    return SkColorFilter::Deserialize(data, length).release();
 }
 
 extern "C" SkColor4f C_SkColorFilter_filterColor4f(
@@ -1794,15 +1802,6 @@ extern "C" SkPathEffect* C_SkPathEffect_MakeCompose(SkPathEffect* outer, SkPathE
     return SkPathEffect::MakeCompose(sp(outer), sp(inner)).release();
 }
 
-extern "C" void C_SkPathEffect_PointData_Construct(SkPathEffect::PointData* uninitialized) {
-    new(uninitialized) SkPathEffect::PointData();
-}
-
-extern "C" void C_SkPathEffect_PointData_deletePoints(SkPathEffect::PointData* self) {
-    delete [] self->fPoints;
-    self->fPoints = nullptr;
-}
-
 extern "C" void C_SkPathEffect_DashInfo_Construct(SkPathEffect::DashInfo* uninitialized) {
     new(uninitialized) SkPathEffect::DashInfo();
 }
@@ -1895,10 +1894,6 @@ extern "C" SkShader* C_SkShaders_Blend(SkBlendMode mode, SkShader* dst, SkShader
     return SkShaders::Blend(mode, sp(dst), sp(src)).release();
 }
 
-extern "C" SkShader* C_SkShaders_Lerp(float t, SkShader* dst, SkShader* src) {
-    return SkShaders::Lerp(t, sp(dst), sp(src)).release();
-}
-
 extern "C" SkShader* C_SkShader_Deserialize(const void* data, size_t length) {
     // note: dynamic_cast may lead to a linker error here on iOS x86_64
     // https://github.com/rust-skia/rust-skia/issues/146
@@ -2094,6 +2089,14 @@ extern "C" bool C_SkYUVAPixmaps_isValid(const SkYUVAPixmaps* self) {
 
 extern "C" void C_Effects_Types(SkTableMaskFilter *) {}
 
+//
+// effects/SkBlenders.h
+//
+
+extern "C" SkBlender* C_SkBlenders_Arithmetic(float k1, float k2, float k3, float k4, bool enforcePremul) {
+    return SkBlenders::Arithmetic(k1, k2, k3, k4, enforcePremul).release();
+}
+
 //
 // effects/SkGradientShader.h
 //
@@ -2280,20 +2283,42 @@ extern "C" SkColorFilter* C_SkOverdrawColorFilter_MakeWithSkColors(const SkColor
 
 extern "C" {
 
-SkRuntimeEffect *C_SkRuntimeEffect_Make(
+SkRuntimeEffect *C_SkRuntimeEffect_MakeForColorFilter(
     const SkString *sksl,
     const SkRuntimeEffect::Options *options,
     SkString *error)
 {
-    auto r = SkRuntimeEffect::Make(*sksl, *options);
+    auto r = SkRuntimeEffect::MakeForColorFilter(*sksl, *options);
     *error = r.errorText;
     return r.effect.release();
 }
 
-SkShader *C_SkRuntimeEffect_makeShader(const SkRuntimeEffect *self, SkData *uniforms, SkShader **children, size_t childCount,
-                                       const SkMatrix *localMatrix, bool isOpaque) {
-    auto childrenSPs = reinterpret_cast<sk_sp<SkShader> *>(children);
-    return self->makeShader(sp(uniforms), childrenSPs, childCount, localMatrix, isOpaque).release();
+SkRuntimeEffect *C_SkRuntimeEffect_MakeForShader(
+    const SkString *sksl,
+    const SkRuntimeEffect::Options *options,
+    SkString *error)
+{
+    auto r = SkRuntimeEffect::MakeForShader(*sksl, *options);
+    *error = r.errorText;
+    return r.effect.release();
+}
+
+SkRuntimeEffect *C_SkRuntimeEffect_MakeForBlender(
+    const SkString *sksl,
+    const SkRuntimeEffect::Options *options,
+    SkString *error)
+{
+    auto r = SkRuntimeEffect::MakeForBlender(*sksl, *options);
+    *error = r.errorText;
+    return r.effect.release();
+}
+
+SkShader *C_SkRuntimeEffect_makeShader(
+    const SkRuntimeEffect *self, SkData *uniforms,
+    SkRuntimeEffect::ChildPtr *children, size_t childCount,
+    const SkMatrix *localMatrix, bool isOpaque)
+{
+    return self->makeShader(sp(uniforms), SkSpan<SkRuntimeEffect::ChildPtr>(children, childCount), localMatrix, isOpaque).release();
 }
 
 SkImage *C_SkRuntimeEffect_makeImage(
@@ -2312,12 +2337,16 @@ SkImage *C_SkRuntimeEffect_makeImage(
         localMatrix, *resultInfo, mipmapped).release();
 }
 
-SkColorFilter* C_SkRuntimeEffect_makeColorFilter(const SkRuntimeEffect* self, SkData* inputs) {
-    return self->makeColorFilter(sp(inputs)).release();
+SkColorFilter *C_SkRuntimeEffect_makeColorFilter(
+    const SkRuntimeEffect *self, SkData *inputs, SkRuntimeEffect::ChildPtr *children, size_t childCount)
+{
+    return self->makeColorFilter(sp(inputs), SkSpan<SkRuntimeEffect::ChildPtr>(children, childCount)).release();
 }
 
-const SkString *C_SkRuntimeEffect_source(const SkRuntimeEffect *self) {
-    return &self->source();
+const unsigned char* C_SkRuntimeEffect_source(const SkRuntimeEffect *self, size_t* len) {
+    auto& str = self->source();
+    *len = str.length();
+    return reinterpret_cast<const unsigned char*>(str.data());
 }
 
 const SkRuntimeEffect::Uniform* C_SkRuntimeEffect_uniforms(const SkRuntimeEffect* self, size_t* count) {
@@ -2326,18 +2355,12 @@ const SkRuntimeEffect::Uniform* C_SkRuntimeEffect_uniforms(const SkRuntimeEffect
     return &*uniforms.begin();
 }
 
-const SkString* C_SkRuntimeEffect_children(const SkRuntimeEffect* self, size_t* count) {
+const SkRuntimeEffect::Child* C_SkRuntimeEffect_children(const SkRuntimeEffect* self, size_t* count) {
     auto children = self->children();
     *count = children.count();
     return &*children.begin();
 }
 
-const SkRuntimeEffect::Varying* C_SkRuntimeEffect_varyings(const SkRuntimeEffect* self, size_t* count) {
-    auto varyings = self->varyings();
-    *count = varyings.count();
-    return &*varyings.begin();
-}
-
 }
 
 //
@@ -2699,23 +2722,15 @@ C_SkCustomTypefaceBuilder_setGlyph3(SkCustomTypefaceBuilder *self, SkGlyphID gly
     self->setGlyph(glyph, advance, sp(picture));
 }
 */
- 
-extern "C" void C_SkInterpolator_destruct(SkInterpolator* self) {
-    self->~SkInterpolator();
-}
-
-extern "C" void C_SkInterpolator_setRepeatCount(SkInterpolator* self, SkScalar repeatCount) {
-    self->setRepeatCount(repeatCount);
-}
-
-extern "C" void C_SkInterpolator_setReset(SkInterpolator* self, bool reset) {
-    self->setReset(reset);
-}
-
-extern "C" void C_SkInterpolator_setMirror(SkInterpolator* self, bool mirror) {
-    self->setMirror(mirror);
-}
 
 extern "C" SkCanvas* C_SkMakeNullCanvas() {
     return SkMakeNullCanvas().release();
 }
+
+extern "C" SkOrderedFontMgr* C_SkOrderedFontMgr_new() {
+    return new SkOrderedFontMgr();
+}
+
+extern "C" void C_SkOrderedFontMgr_append(SkOrderedFontMgr* self, SkFontMgr* fontMgr) {
+    self->append(sp(fontMgr));
+}
